@@ -27,7 +27,6 @@
 #include "download.h"
 
 static CURLM *multi_handle=NULL;
-static int still_running=0;
 struct download_info {
     CURL *handle;
     FILE *fp;
@@ -86,7 +85,7 @@ static void handle_done(CURLMsg *msg){
         if(msg->data.result==CURLE_HTTP_RETURNED_ERROR){
             if(curl_easy_getinfo(info->handle, CURLINFO_RESPONSE_CODE, &status)!=CURLE_OK) status=600;
             if(status!=404 || !(info->flags&DOWNLOAD_NO_404))
-                warn("HTTP download of %s returned %d", info->filename, status);
+                warn("HTTP download of %s returned %ld", info->filename, status);
         } else {
             warn("Download of %s failed: %s", info->filename, curl_easy_strerror(msg->data.result));
         }
@@ -115,12 +114,26 @@ void download_process(unsigned long sleeptime){
     struct timeval tv;
     int maxfd, n, x;
     CURLMsg *msg;
+    CURLMcode status;
 
-    if(sleeptime>0){
+    if(sleeptime>0) do {
         FD_ZERO(&rd);
         FD_ZERO(&wr);
         FD_ZERO(&er);
-        curl_multi_fdset(multi_handle, &rd, &wr, &er, &maxfd);
+        errno = 0;
+        status = curl_multi_fdset(multi_handle, &rd, &wr, &er, &maxfd);
+        if(status!=CURLM_OK){
+            warn("Could not fetch curl fdset: %s", curl_multi_strerror(status));
+            usleep(sleeptime);
+            return;
+        }
+        if(maxfd<0){
+            /* It's not something we can select on, sleep and then blindly call
+             * curl_multi_perform.
+             */
+            usleep(sleeptime);
+            break;
+        }
 
         tv.tv_sec=0;
         tv.tv_usec=sleeptime;
@@ -145,7 +158,8 @@ void download_process(unsigned long sleeptime){
             usleep(sleeptime);
             return;
         }
-    }
+    } while(0);
+
     while(curl_multi_perform(multi_handle, &x)==CURLM_CALL_MULTI_PERFORM);
     while((msg=curl_multi_info_read(multi_handle, &x))){
         switch(msg->msg){
@@ -176,7 +190,6 @@ int download_kill(char *filename){
 
 int download_file(char *filename, char *from_addr, char *postdata, int flags, void (*callback)(char *filename, void *data), void *data){
     struct download_info *info=NULL;
-    FILE *fp;
 
     if(callback==NULL || filename==NULL || from_addr==NULL) return 1;
 
@@ -229,8 +242,7 @@ int download_file(char *filename, char *from_addr, char *postdata, int flags, vo
         goto fail;
     }
     if(postdata!=NULL){
-        if(curl_easy_setopt(info->handle, CURLOPT_COPYPOSTFIELDS, postdata)!=CURLE_OK
-          ){
+        if(curl_easy_setopt(info->handle, CURLOPT_COPYPOSTFIELDS, postdata)!=CURLE_OK){
             warn("Error setting CURL post options");
             goto fail;
         }
